---------------------------------------------------------
--! @file hchecker_tb.vhd
--! @brief Testbench for Distance-3 Hamming Code Checker
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2019-04-09
---------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

entity hchecker_tb is
end entity hchecker_tb;

architecture arch of hchecker_tb is

-- Component Declaration
  component hchecker is    
    port (
      DIN :  in std_logic_vector  (7 downto 1); -- Data in      
      DOUT : out std_logic_vector (7 downto 1); -- Data out
      EBIT : out std_logic_vector (2 downto 0); -- Bit Number      
      ERR :  out std_logic                      -- Error      
      );    
end component hchecker;

-- Signals
signal din :  std_logic_vector (7 downto 1);
signal dout : std_logic_vector (7 downto 1);
signal ebit : std_logic_vector (2 downto 0);
signal err :  std_logic;

begin
  -- Component Instantiation
  HC : component hchecker port map (din,dout,ebit,err);
    
  -- Stimulus process
  stimulus_process : process is
    type pattern_type is record
     -- The inputs of the circuit
      din : std_logic_vector (7 downto 1);
     -- The expected outputs of the circuit
      dout : std_logic_vector (7 downto 1);
      ebit : std_logic_vector (2 downto 0);      
      err :  std_logic;
    end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (("0000000","0000000","000",'0'), 
       ("0000110","0000111","001",'1'),
       ("0011011","0011001","010",'1'),
       ("0011010","0011110","011",'1'),
       ("0100010","0101010","100",'1'),
       ("0111101","0101101","101",'1'),
       ("0010011","0110011","110",'1'),
       ("1110100","0110100","111",'1'),
       ("1101011","1001011","110",'1'),
       ("1011100","1001100","101",'1'),
       ("1011010","1010010","100",'1'),
       ("1010001","1010101","011",'1'),
       ("1100011","1100001","010",'1'),
       ("1100111","1100110","001",'1'),
       ("1111000","1111000","000",'0'),
       ("0111111","1111111","111",'1'));

    begin

    wait for 1 ns;
      
    --  Check each pattern.
    for k in patterns'range loop
      --  Set the inputs.
       din <= patterns(k).din;
      
    --  Wait for the results.
    wait for 3 ns;
    --  Check the outputs.
    assert dout = patterns(k).dout
      report "bad check DOUT" severity error;
    assert ebit = patterns(k).ebit
      report "bad check EBIT" severity error;
    assert err = patterns(k).err
      report "bad check ERR" severity error;         
    end loop;
    assert false report "end of test" severity note;
    --  Wait forever; this will finish the simulation.
      wait;
 end process;

end architecture arch;
