-------------------------------------------------------
--! @file hgenerator_tb.vhd
--! @brief Testbench for 7-bit Hamming Code Generator
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2019-04-09
-------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

entity hgenerator_tb is
end entity hgenerator_tb;

architecture arch of hgenerator_tb is

-- Component Declaration
component hgenerator is
  port (
    DIN :    in std_logic_vector  (3 downto 0);  -- Data in
    DOUT :   out std_logic_vector (6 downto 0)   -- Data out
    );
end component hgenerator;

-- Signals
signal din :  std_logic_vector (3 downto 0);
signal dout : std_logic_vector (6 downto 0);

begin
  -- Component Instantiation
  HG : component hgenerator port map (din,dout);
    
  -- Stimulus process
  stimulus_process : process is
    type pattern_type is record
     -- The inputs of the circuit
     din : std_logic_vector (3 downto 0);
     -- The expected outputs of the circuit
     dout : std_logic_vector (6 downto 0);
     end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (("0000","0000000"), 
       ("0001","0000111"),
       ("0010","0011001"),
       ("0011","0011110"),
       ("0100","0101010"),
       ("0101","0101101"),
       ("0110","0110011"),
       ("0111","0110100"),
       ("1000","1001011"),
       ("1001","1001100"),
       ("1010","1010010"),
       ("1011","1010101"),
       ("1100","1100001"),
       ("1101","1100110"),
       ("1110","1111000"),
       ("1111","1111111"));

    begin

    wait for 1 ns;
      
    --  Check each pattern.
    for k in patterns'range loop
      --  Set the inputs.
      din <= patterns(k).din;
  
    --  Wait for the results.
    wait for 3 ns;
    --  Check the outputs.
    assert dout = patterns(k).dout
      report "bad check DOUT" severity error;
     end loop;
    assert false report "end of test" severity note;
    --  Wait forever; this will finish the simulation.
      wait;
 end process;

end architecture arch;
