-------------------------------------------------------
--! @file encoder_4to2_tb.vhd
--! @brief Testbench for 4-to-2 encoder
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2020-04-01
-------------------------------------------------------

entity encoder_4to2_tb is
end entity encoder_4to2_tb;

architecture testbench of encoder_4to2_tb is

--  Declaration of the component that will be instantiated.  
  component encoder_4to2
  port (
    A : in bit_vector (3 downto 0);
    Y : out bit_vector (1 downto 0)
    );
  end component encoder_4to2;

  signal a : bit_vector(3 downto 0);
  signal y : bit_vector(1 downto 0);
  
begin
  --  Component instantiation.
  DUT: entity work.encoder_4to2 (with_select) port map (
    A => a,
    Y => y
    );

  --  This process does the real job.
  stimulus_process: process is
    type pattern_type is record
      --  The inputs of the circuit.
      a : bit_vector (3 downto 0);
      
      --  The expected outputs of the circuit.
      y : bit_vector (1 downto 0);
    end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (
        ("0000","00"),
        ("0001","00"),
        ("0010","01"),
        ("0011","00"),
        ("0100","10"),
        ("0101","00"),
        ("0110","00"),
        ("0111","00"),
        ("1000","11"),
        ("1001","00"),
        ("1010","00"),
        ("1011","00"),
        ("1100","00"),
        ("1101","00"),
        ("1110","00"),
        ("1111","00"));
        
  begin
    for k in patterns'range loop

      --  Set the inputs.
      a <= patterns(k).a;
      
       --  Wait for the results.
      wait for 1 ns;
      
  --  Check the outputs.

      assert y = patterns(k).y        
        report "bad y" severity error;

    end loop;
    
    assert false report "end of test" severity note;
    
  --  Wait forever; this will finish the simulation.
    wait;
  end process;
  
end architecture testbench;
  

