-------------------------------------------------------
--! @file adder4_tb.vhd
--! @brief Testbench for 4-bit adder
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2018-10-02
-------------------------------------------------------

entity adder4_tb is
end entity adder4_tb;

architecture arch of adder4_tb is

-- Component Declaration
component adder4 is
  port (
    cin : in bit;
    x : in bit_vector (3 downto 0);
    y : in bit_vector (3 downto 0);
    sum : out bit_vector (3 downto 0);
    cout : out bit
    );
end component adder4;

-- Signals
signal cin : bit;
signal x : bit_vector (3 downto 0);
signal y : bit_vector (3 downto 0);
signal sum : bit_vector (3 downto 0);
signal cout : bit;

begin
  -- Component Instantiation
  UADDER : component adder4 port map (
    cin => cin,
    x => x,
    y => y,
    sum => sum,
    cout => cout
    );
    
  -- Stimulus process
  stimulus_process : process is
    type pattern_type is record
     -- The inputs of the circuit
     cin : bit;
     x : bit_vector (3 downto 0);
     y : bit_vector (3 downto 0); 
     -- The expected outputs of the circuit
     sum : bit_vector (3 downto 0);
     cout : bit;
     end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (('0',"1010","0101","1111",'0'),
       ('1',"1010","0101","0000",'1'),
       ('0',"0111","0001","1000",'0'),
       ('1',"0111","0001","1001",'0'),
       ('0',"1000","1000","0000",'1'),
       ('1',"1000","1000","0001",'1'));

    begin

    wait for 1 ns;
      
    --  Check each pattern.
    for k in patterns'range loop
      --  Set the inputs.
      cin <= patterns(k).cin;
      x <= patterns(k).x;
      y <= patterns(k).y;
  
    --  Wait for the results.
    wait for 3 ns;
    --  Check the outputs.
    assert sum = patterns(k).sum
      report "bad check" severity error;
    assert cout = patterns(k).cout
      report "bad check" severity error;
    end loop;
    assert false report "end of test" severity note;
    --  Wait forever; this will finish the simulation.
      wait;
 end process;

end architecture arch;
