-------------------------------------------------------
--! @file comps4_tb.vhd
--! @brief Testbench 4-bit signed comparator x>y
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2018-10-03
-------------------------------------------------------

entity comps4_tb is
end entity comps4_tb;

architecture arch of comps4_tb is

-- Component Declaration
component comps4 is
  port (
    x : in bit_vector (3 downto 0);
    y : in bit_vector (3 downto 0);
    xgty : out bit
    );
end component comps4;

-- Signals
signal x : bit_vector (3 downto 0);
signal y : bit_vector (3 downto 0);
signal xgty : bit;

begin
  -- Component Instantiation
  UCOMPS4 : component comps4 port map
    (x,y,xgty);
    
  -- Stimulus process
  stimulus_process : process is
    type pattern_type is record
     -- The inputs of the circuit
     x : bit_vector (3 downto 0);
     y : bit_vector (3 downto 0); 
     -- The expected outputs of the circuit
     xgty : bit;
     end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (("0001","1111",'1'),
       ("0111","1000",'1'),
       ("1101","0110",'0'),
       ("1111","1110",'1'),
       ("0111","1001",'1'));

   begin

    wait for 1 ns;
      
    --  Check each pattern.
    for k in patterns'range loop
      --  Set the inputs.
       x <= patterns(k).x;
      y <= patterns(k).y;
  
    --  Wait for the results.
    wait for 3 ns;
    --  Check the outputs.
    assert xgty = patterns(k).xgty
      report "bad check" severity error;
    end loop;
    assert false report "end of test" severity note;
    --  Wait forever; this will finish the simulation.
      wait;
 end process;

end architecture arch;
