-------------------------------------------------------
--! @file twocomp4_tb.vhd
--! @brief Testbench for 4-bit 2's Complement
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2018-10-07
-------------------------------------------------------

entity twocomp4_tb is
end entity twocomp4_tb;

architecture arch of twocomp4_tb is

-- Component Declaration
component twocomp4 is
port (
    A : in bit_vector (3 downto 0);
    A2COMP : out bit_vector (3 downto 0)
    );
end component twocomp4;

-- Signals
signal a : bit_vector (3 downto 0);
signal a2comp : bit_vector (3 downto 0);

begin
  -- Component Instantiation
  U2COMP4 : component twocomp4 port map (a,a2comp);
    
  -- Stimulus process
  stimulus_process : process is
    type pattern_type is record
     -- The inputs of the circuit
     a : bit_vector (3 downto 0);
     -- The expected outputs of the circuit
     a2comp : bit_vector (3 downto 0);
     end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (("0000","0000"),
       ("0001","1111"),
       ("0010","1110"),
       ("0011","1101"),
       ("0100","1100"),
       ("0101","1011"),
       ("0110","1010"),
       ("0111","1001"),
       ("1000","1000"),
       ("1001","0111"),
       ("1010","0110"),
       ("1011","0101"),
       ("1100","0100"),
       ("1101","0011"),
       ("1110","0010"),
       ("1111","0001"));
       
    begin

    wait for 1 ns;
      
    --  Check each pattern.
    for k in patterns'range loop
      --  Set the inputs.
      a <= patterns(k).a;
  
    --  Wait for the results.
    wait for 3 ns;
    --  Check the outputs.
    assert a2comp = patterns(k).a2comp
      report "bad check" severity error;
    end loop;
    assert false report "end of test" severity note;
    --  Wait forever; this will finish the simulation.
      wait;
 end process;

end architecture arch;
